package java.util;

import java.io.ObjectInputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.io.Serializable;

public class TreeSet<E> extends AbstractSet<E> implements NavigableSet<E>, Cloneable, Serializable
{
    private transient NavigableMap<E, Object> m;
    private static final Object PRESENT;
    private static final long serialVersionUID = -2479143000061671589L;
    
    TreeSet(NavigableMap<E, Object> m) {
        this.m = m;
    }
    
    public TreeSet() {
        this(new TreeMap());
    }
    
    public TreeSet(Comparator<? super E> comparator) {
        this(new TreeMap(comparator));
    }
    
    public TreeSet(Collection<? extends E> collection) {
        this();
        this.addAll(collection);
    }
    
    public TreeSet(SortedSet<E> set) {
        this(set.comparator());
        this.addAll(set);
    }
    
    @Override
    public Iterator<E> iterator() {
        return this.m.navigableKeySet().iterator();
    }
    
    @Override
    public Iterator<E> descendingIterator() {
        return this.m.descendingKeySet().iterator();
    }
    
    @Override
    public NavigableSet<E> descendingSet() {
        return new TreeSet(this.m.descendingMap());
    }
    
    @Override
    public int size() {
        return this.m.size();
    }
    
    @Override
    public boolean isEmpty() {
        return this.m.isEmpty();
    }
    
    @Override
    public boolean contains(Object o) {
        return this.m.containsKey(o);
    }
    
    @Override
    public boolean add(E e) {
        return this.m.put(e, TreeSet.PRESENT) == null;
    }
    
    @Override
    public boolean remove(Object o) {
        return this.m.remove(o) == TreeSet.PRESENT;
    }
    
    @Override
    public void clear() {
        this.m.clear();
    }
    
    @Override
    public boolean addAll(Collection<? extends E> collection) {
        if (this.m.size() == 0 && collection.size() > 0 && collection instanceof SortedSet && this.m instanceof TreeMap) {
            SortedSet<Object> set = (SortedSet)collection;
            TreeMap<Object, TreeSet> treeMap = (TreeMap)this.m;
            Comparator<Object> comparator = set.comparator();
            Comparator<Object> comparator2 = treeMap.comparator();
            if (comparator == comparator2 || (comparator != null && comparator.equals(comparator2))) {
                treeMap.addAllForTreeSet(set, TreeSet.PRESENT);
                return true;
            }
        }
        return super.addAll(collection);
    }
    
    @Override
    public NavigableSet<E> subSet(E e, boolean b, E e2, boolean b2) {
        return new TreeSet(this.m.subMap(e, b, e2, b2));
    }
    
    @Override
    public NavigableSet<E> headSet(E e, boolean b) {
        return new TreeSet(this.m.headMap(e, b));
    }
    
    @Override
    public NavigableSet<E> tailSet(E e, boolean b) {
        return new TreeSet(this.m.tailMap(e, b));
    }
    
    @Override
    public SortedSet<E> subSet(E e, E e2) {
        return this.subSet(e, true, e2, false);
    }
    
    @Override
    public SortedSet<E> headSet(E e) {
        return this.headSet(e, false);
    }
    
    @Override
    public SortedSet<E> tailSet(E e) {
        return this.tailSet(e, true);
    }
    
    @Override
    public Comparator<? super E> comparator() {
        return this.m.comparator();
    }
    
    @Override
    public E first() {
        return this.m.firstKey();
    }
    
    @Override
    public E last() {
        return this.m.lastKey();
    }
    
    @Override
    public E lower(E e) {
        return this.m.lowerKey(e);
    }
    
    @Override
    public E floor(E e) {
        return this.m.floorKey(e);
    }
    
    @Override
    public E ceiling(E e) {
        return this.m.ceilingKey(e);
    }
    
    @Override
    public E higher(E e) {
        return this.m.higherKey(e);
    }
    
    @Override
    public E pollFirst() {
        Map.Entry<E, Object> pollFirstEntry = this.m.pollFirstEntry();
        return (pollFirstEntry == null) ? null : pollFirstEntry.getKey();
    }
    
    @Override
    public E pollLast() {
        Map.Entry<E, Object> pollLastEntry = this.m.pollLastEntry();
        return (pollLastEntry == null) ? null : pollLastEntry.getKey();
    }
    
    public Object clone() {
        TreeSet<Object> set;
        try {
            set = (TreeSet)super.clone();
        }
        catch (CloneNotSupportedException cause) {
            throw new InternalError(cause);
        }
        set.m = new TreeMap(this.m);
        return set;
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        objectOutputStream.defaultWriteObject();
        objectOutputStream.writeObject(this.m.comparator());
        objectOutputStream.writeInt(this.m.size());
        Iterator<Object> iterator = this.m.keySet().iterator();
        while (iterator.hasNext()) {
            objectOutputStream.writeObject(iterator.next());
        }
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        objectInputStream.defaultReadObject();
        (this.m = new TreeMap((Comparator)objectInputStream.readObject())).readTreeSet(objectInputStream.readInt(), objectInputStream, TreeSet.PRESENT);
    }
    
    @Override
    public Spliterator<E> spliterator() {
        return TreeMap.keySpliteratorFor(this.m);
    }
    
    static {
        PRESENT = new Object();
    }
}
